@@ -1,5 +1,6 @@
 from importlib import import_module
 from typing import Callable
+from typing import TypeVar
 
 
 def load_handler(handler_id: str) -> Callable:
@@ -7,3 +8,8 @@ def load_handler(handler_id: str) -> Callable:
     module = import_module(".".join(module_path_elements))
 
     return getattr(module, object_name)
+
+
+T = TypeVar("T")
+Func = Callable[..., T]
+Decorator = Callable[[Func[T]], Func[T]]
