@@ -1,9 +1,10 @@
 from functools import partial
 from functools import wraps
-from typing import Callable
+from typing import Any
 from typing import Optional
 from typing import Sequence
 from typing import Type
+from typing import TypeVar
 
 import jsonschema
 from flask import Request
@@ -12,87 +13,72 @@ from flask import request as current_flask_request
 from asynction.exceptions import BindingsValidationException
 from asynction.exceptions import MessageAckValidationException
 from asynction.exceptions import PayloadValidationException
+from asynction.exceptions import ValidationException
 from asynction.types import ChannelBindings
 from asynction.types import JSONMapping
 from asynction.types import JSONSchema
 from asynction.types import Message
 from asynction.types import MessageAck
+from asynction.utils import Decorator
+from asynction.utils import Func
 
 
-def jsonschema_validate_with_error_handling(
-    instance: JSONMapping, schema: JSONSchema, exc_type: Type[Exception]
+def jsonschema_validate_with_custom_error(
+    instance: JSONMapping, schema: JSONSchema, exc_type: Type[ValidationException]
 ) -> None:
     try:
         jsonschema.validate(instance, schema)
     except jsonschema.ValidationError as e:
-        raise exc_type(str(e))
+        raise exc_type.create_from(e)
 
 
 jsonschema_validate_payload = partial(
-    jsonschema_validate_with_error_handling,
+    jsonschema_validate_with_custom_error,
     exc_type=PayloadValidationException,
 )
 
 jsonschema_validate_bindings = partial(
-    jsonschema_validate_with_error_handling, exc_type=BindingsValidationException
+    jsonschema_validate_with_custom_error, exc_type=BindingsValidationException
 )
 
 jsonschema_validate_ack = partial(
-    jsonschema_validate_with_error_handling, exc_type=MessageAckValidationException
+    jsonschema_validate_with_custom_error, exc_type=MessageAckValidationException
 )
 
 
 def validate_payload(args: Sequence, schema: Optional[JSONSchema]) -> None:
     if schema is None:
-        if args:
-            raise PayloadValidationException(
-                "Handler args provided for message that has no payload defined."
-            )
-        # Validation succeeded since there is no message payload specified
-        # and no args have been provided.
+        # Validation skipped since there is no message payload specified
         return
 
-    if schema["type"] == "array" and schema.get("prefixItems"):  # Tuple validation
+    # TODO: This check should be driven by the schema rather than the input args
+    if len(args) > 1:
         jsonschema_validate_payload(list(args), schema)
     else:
-        if len(args) > 1:
-            raise PayloadValidationException(
-                "Multiple handler arguments provided, "
-                f"although schema type is: {schema['type']}"
-            )
-
         jsonschema_validate_payload(args[0], schema)
 
 
 def validate_ack_args(args: Sequence, message_ack_spec: Optional[MessageAck]) -> None:
     if message_ack_spec is None:
-        if args:
-            raise MessageAckValidationException(
-                "Callback args provided for message that has no x-ack schema specified."
-            )
-        # Validation succeeded since there is no message ack specified
-        # and no args have been provided.
+        # Validation skipped since there is no message ack specified
         return
 
-    schema_type = message_ack_spec.args["type"]
-    if schema_type == "array" and message_ack_spec.args.get("prefixItems"):  # Tuple
+    # TODO: This check should be driven by the schema rather than the input args
+    if len(args) > 1:
         jsonschema_validate_ack(list(args), message_ack_spec.args)
     else:
-        if len(args) > 1:
-            raise MessageAckValidationException(
-                "Multiple callback arguments provided, "
-                f"although x-ack schema type is: {schema_type}"
-            )
-
         jsonschema_validate_ack(args[0], message_ack_spec.args)
 
 
-def publish_message_validator_factory(message: Message) -> Callable:
+T = TypeVar("T")
+
+
+def publish_message_validator_factory(message: Message) -> Decorator[T]:
     """Constructs a validating wrapper for any incoming (`publish`) message handler"""
 
-    def decorator(handler: Callable):
+    def decorator(handler: Func[T]) -> Func[T]:
         @wraps(handler)
-        def handler_with_validation(*args, **kwargs):
+        def handler_with_validation(*args: Any, **kwargs: Any) -> T:
             validate_payload(args, message.payload)
 
             ack = handler(*args, **kwargs)
@@ -108,10 +94,10 @@ def publish_message_validator_factory(message: Message) -> Callable:
     return decorator
 
 
-def callback_validator_factory(message: Message) -> Callable:
-    def decorator(callback: Callable):
+def callback_validator_factory(message: Message) -> Decorator[T]:
+    def decorator(callback: Func[T]) -> Func[T]:
         @wraps(callback)
-        def callback_with_validation(*args):
+        def callback_with_validation(*args: Any) -> T:
             # the callback should only be called with positional arguments
             validate_ack_args(args, message.x_ack)
             return callback(*args)
@@ -143,10 +129,10 @@ def validate_request_bindings(
         jsonschema_validate_bindings(request.args.to_dict(), bindings.ws.query)
 
 
-def bindings_validator_factory(bindings: Optional[ChannelBindings]) -> Callable:
-    def decorator(handler: Callable):
+def bindings_validator_factory(bindings: Optional[ChannelBindings]) -> Decorator[T]:
+    def decorator(handler: Func[T]) -> Func[T]:
         @wraps(handler)
-        def handler_with_validation(*args, **kwargs):
+        def handler_with_validation(*args: Any, **kwargs: Any) -> T:
             validate_request_bindings(current_flask_request, bindings)
             return handler(*args, **kwargs)
 
